@@ -1,5 +1,5 @@
 /* idn2.c - command line interface to libidn2
-   Copyright (C) 2011-2019 Simon Josefsson, Tim Ruehsen
+   Copyright (C) 2011-2021 Simon Josefsson, Tim Ruehsen
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ const char version_etc_copyright[] =
   /* Do *not* mark this string for translation.  %s is a copyright
      symbol suitable for this locale, and %d is the copyright
      year.  */
-  "Copyright 2011-%s %d Simon Josefsson, Tim Ruehsen.";
+  "Copyright %s 2011-%d Simon Josefsson, Tim Ruehsen.";
 
 static void
 usage (int status)
@@ -93,7 +93,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
 "), stdout);
       fputs (_("\
       --usestd3asciirules   Enable STD3 ASCII rules\n\
-      --no-alabelroundtrip  Disable ALabel rountrip for lookups\n\
+      --no-alabelroundtrip  Disable A-label roundtrip for lookups\n\
       --debug               Print debugging information\n\
       --quiet               Silent operation\n\
 "), stdout);
