@@ -1,5 +1,5 @@
 /* test-punycode.c --- Self tests for Libidn2 punycode.
-   Copyright (C) 2002-2017 Simon Josefsson
+   Copyright (C) 2002-2021 Simon Josefsson
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -167,7 +167,7 @@ static int debug = 0;
 static int error_count = 0;
 static int break_on_error = 0;
 
-_GL_ATTRIBUTE_FORMAT_PRINTF (1, 2) static void
+_GL_ATTRIBUTE_FORMAT_PRINTF_STANDARD (1, 2) static void
 fail (const char *format, ...)
 {
   va_list arg_ptr;
@@ -236,8 +236,8 @@ main (void)
 	}
 
       outlen = BUFSIZ;
-      rc = _idn2_punycode_encode (
-	punycode[i].inlen, punycode[i].in, &outlen, p);
+      rc = _idn2_punycode_encode_internal (punycode[i].inlen, punycode[i].in,
+					   &outlen, p);
       if (rc != punycode[i].rc)
 	{
 	  fail ("punycode_encode() entry %d failed: %d\n", (int) i, rc);
@@ -278,8 +278,8 @@ main (void)
 	}
 
       outlen = BUFSIZ;
-      rc = _idn2_punycode_decode (strlen (punycode[i].out), punycode[i].out,
-				    &outlen, q);
+      rc = _idn2_punycode_decode_internal (strlen (punycode[i].out),
+					   punycode[i].out, &outlen, q);
       if (rc != punycode[i].rc)
 	{
 	  fail ("punycode() entry %d failed: %d\n", (int) i, rc);
