@@ -1,5 +1,5 @@
 /* Character set conversion with error handling.
-   Copyright (C) 2001-2019 Free Software Foundation, Inc.
+   Copyright (C) 2001-2021 Free Software Foundation, Inc.
    Written by Bruno Haible and Simon Josefsson.
 
    This program is free software: you can redistribute it and/or
@@ -508,11 +508,7 @@ mem_cd_iconveh_internal (const char *src, size_t srclen,
             else
               {
                 if (result != initial_result)
-                  {
-                    int saved_errno = errno;
-                    free (result);
-                    errno = saved_errno;
-                  }
+                  free (result);
                 return -1;
               }
           }
@@ -579,11 +575,7 @@ mem_cd_iconveh_internal (const char *src, size_t srclen,
           else
             {
               if (result != initial_result)
-                {
-                  int saved_errno = errno;
-                  free (result);
-                  errno = saved_errno;
-                }
+                free (result);
               return -1;
             }
         }
@@ -692,11 +684,7 @@ mem_cd_iconveh_internal (const char *src, size_t srclen,
             && !(errno == E2BIG || errno == EINVAL || errno == EILSEQ))
           {
             if (result != initial_result)
-              {
-                int saved_errno = errno;
-                free (result);
-                errno = saved_errno;
-              }
+              free (result);
             return -1;
           }
         if (res1 == (size_t)(-1)
@@ -916,22 +904,14 @@ mem_cd_iconveh_internal (const char *src, size_t srclen,
                           {
                             /* Failure converting the ASCII replacement.  */
                             if (result != initial_result)
-                              {
-                                int saved_errno = errno;
-                                free (result);
-                                errno = saved_errno;
-                              }
+                              free (result);
                             return -1;
                           }
                       }
                     else
                       {
                         if (result != initial_result)
-                          {
-                            int saved_errno = errno;
-                            free (result);
-                            errno = saved_errno;
-                          }
+                          free (result);
                         return -1;
                       }
                   }
@@ -1050,12 +1030,7 @@ str_cd_iconveh (const char *src,
 
   if (retval < 0)
     {
-      if (result != NULL)
-        {
-          int saved_errno = errno;
-          free (result);
-          errno = saved_errno;
-        }
+      free (result);
       return NULL;
     }
 
@@ -1127,12 +1102,8 @@ mem_iconveh (const char *src, size_t srclen,
         {
           if (iconveh_close (&cd) < 0)
             {
-              /* Return -1, but free the allocated memory, and while doing
-                 that, preserve the errno from iconveh_close.  */
-              int saved_errno = errno;
-              if (result != *resultp && result != NULL)
+              if (result != *resultp)
                 free (result);
-              errno = saved_errno;
               return -1;
             }
           *resultp = result;
@@ -1186,11 +1157,7 @@ str_iconveh (const char *src,
         {
           if (iconveh_close (&cd) < 0)
             {
-              /* Return NULL, but free the allocated memory, and while doing
-                 that, preserve the errno from iconveh_close.  */
-              int saved_errno = errno;
               free (result);
-              errno = saved_errno;
               return NULL;
             }
         }
