@@ -1,5 +1,5 @@
 /* Store a character in UTF-32 string.
-   Copyright (C) 2002, 2005-2006, 2009-2019 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2005-2006, 2009-2021 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2002.
 
    This program is free software: you can redistribute it and/or
@@ -37,7 +37,7 @@
 #if !HAVE_INLINE
 
 int
-u32_uctomb (uint32_t *s, ucs4_t uc, int n)
+u32_uctomb (uint32_t *s, ucs4_t uc, ptrdiff_t n)
 {
   if (uc < 0xd800 || (uc >= 0xe000 && uc < 0x110000))
     {
